<?php

require __DIR__ . "/vendor/autoload.php";
require __DIR__ . "/vendor/zeedhi/framework/bootstrap.php";
require __DIR__ . "/vendor/teknisa/libraries/scripts/bootstrap.php";

use Doctrine\DBAL\Types\Type;

/**
 * @todo remove setforms2 entities and classes
 */

$instanceManager = \Zeedhi\Framework\DependencyInjection\InstanceManager::getInstance();
$routes = array();

$mapping = getMapping();
loadConfigFiles($instanceManager);
configAcl($instanceManager, $routes);
configLog($instanceManager, $routes);
loadRoutesAndServices($mapping, $instanceManager, $routes);

$instanceManager->compile();

useSetform2Classes($instanceManager);

function getMapping() {
	$mapping = file_get_contents(__DIR__ . "/config/servicesMapping.json");
	return json_decode($mapping, true);
}

function loadRoutesAndServices($mapping, $instanceManager, &$routes) {
	$routes[] = __DIR__ . "/vendor/teknisa/libraries/libRoutes.json";
	$routes[] = __DIR__ . "/vendor/zeedhi/preferences/routes.json";
	foreach ($mapping as $path) {
		$ext = getFilePathExtension($path);
		if($ext === 'xml')
			$instanceManager->loadFromFile(__DIR__ . $path);
		if($ext === 'json')
			$routes[] = __DIR__ . $path;
	}

	$instanceManager->registerService("routeParser", new \Zeedhi\Framework\Routing\Parsers\JSONFormat($routes));
}

function loadConfigFiles($instanceManager) {
	$db 	= __DIR__ . "/../../../backend/config/db.xml"; // prioritize product database config
	$report = __DIR__ . "/config/report.xml"; // prioritize microservice report config
	$parameters = __DIR__ . "/../../../backend/config/parameters.xml"; // prioritize product parameters
	$general = __DIR__ . "/../../../backend/config/general.xml"; // prioritize product general services

	if(!realpath($db))
		$db = __DIR__ . "/config/db.xml";

	if(!realpath($report))
		$report = __DIR__ . "/../../../backend/config/report.xml";

	if (realpath($parameters))
		$instanceManager->loadFromFile($parameters);

	if (realpath($general))
		$instanceManager->loadFromFile($general);

	$instanceManager->loadFromFile(__DIR__ . "/config/environment.xml");
	$instanceManager->loadFromFile(__DIR__ . "/config/fileUploader.xml");
	$instanceManager->loadFromFile($db);
	$instanceManager->loadFromFile($report);
	$instanceManager->loadFromFile(__DIR__ . "/vendor/zeedhi/preferences/services.xml");
}

function configAcl($instanceManager, &$routes) {
	$container = $instanceManager->getContainer();
	$aclParam = "use_zhu_acl";

	if ($container->hasParameter($aclParam) && $container->getParameter($aclParam)) {
		$acl = __DIR__ . "/config/acl.xml";

		if(realpath($acl))
			$instanceManager->loadFromFile($acl);

		require __DIR__ . "/vendor/zeedhi/zhuaclbackend/service/bootstrap.php";
		$routes[] = __DIR__ . "/vendor/zeedhi/zhuaclbackend/config/routes.json";
	}
}

function configLog($instanceManager, &$routes) {
	$container = $instanceManager->getContainer();
	$logParam = "use_zhu_log";
	if ($container->hasParameter($logParam) && $container->getParameter($logParam)) {
    $zhuDateFormat = $container->getParameter('zhuLogDateFormat');
    $ignoredRoutes = $container->hasParameter('logIgnoredRoutes') ? $container->getParameter('logIgnoredRoutes') : [];

    require __DIR__ . "/vendor/zeedhi/zhulog/config/bootstrap.php";
    $routes[] = __DIR__."/vendor/zeedhi/zhulog/config/routes.json";

    $log = __DIR__.'/../../acl/backend/config/log.xml';
    if(realpath($log)){
		$instanceManager->loadFromFile($log);
    }

	$libLog = __DIR__.'/vendor/teknisa/libraries/zhuLog.xml';
    if(realpath($libLog)){
        $instanceManager->loadFromFile($libLog);
    }

    $container->setParameter('logIgnoredRoutes', $ignoredRoutes);
    $container->setParameter('zhuLogDateFormat', $zhuDateFormat);
}
}

function useSetform2Classes($instanceManager) {
	Type::overrideType(Type::DATE, 'Setforms2\Shared\ORM\DateType');
	Type::overrideType(Type::DATETIME, 'Setforms2\Shared\ORM\DateTimeType');

	$entity_manager = $instanceManager->getService('entityManager');
	\Setforms2\Shared\Registry::store("EntityManager", $entity_manager);
	\Setforms2\Shared\Registry::store("EntityManagerConnection", $entity_manager->getConnection());
}

function getFilePathExtension($path) {
	return substr(strrchr($path, '.'), 1);
}